"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.makeGrafastSchema = void 0;
const tslib_1 = require("tslib");
const graphql = tslib_1.__importStar(require("graphql"));
const utils_js_1 = require("./utils.js");
const { buildASTSchema, isEnumType, isInputObjectType, isInterfaceType, isObjectType, isScalarType, isUnionType, parse, } = graphql;
/**
 * Takes a GraphQL schema definition in Interface Definition Language (IDL/SDL)
 * syntax and configs for the types in it and returns a GraphQL schema.
 */
function makeGrafastSchema(details) {
    const { typeDefs, plans, enableDeferStream = false } = details;
    const schema = buildASTSchema(parse(typeDefs), {
        enableDeferStream,
    });
    // Now add the plans/etc to the schema
    for (const [typeName, spec] of Object.entries(plans)) {
        const type = schema.getType(typeName);
        if (!type) {
            console.warn(`'plans' specified configuration for type '${typeName}', but that type was not present in the schema`);
            continue;
        }
        if (isObjectType(type)) {
            if (typeof spec !== "object" || !spec) {
                throw new Error(`Invalid object config for '${typeName}'`);
            }
            const objSpec = spec;
            const fields = type.getFields();
            for (const [fieldName, fieldSpec] of Object.entries(objSpec)) {
                if (fieldName === "__assertStep") {
                    (0, utils_js_1.exportNameHint)(fieldSpec, `${typeName}_assertStep`);
                    type.extensions.grafast = { assertStep: fieldSpec };
                    continue;
                }
                else if (fieldName.startsWith("__")) {
                    throw new Error(`Unsupported field name '${fieldName}'; perhaps you meant '__assertStep'?`);
                }
                const field = fields[fieldName];
                if (!field) {
                    console.warn(`'plans' specified configuration for object type '${typeName}' field '${fieldName}', but that field was not present in the type`);
                    continue;
                }
                if (typeof fieldSpec === "function") {
                    (0, utils_js_1.exportNameHint)(fieldSpec, `${typeName}_${fieldName}_plan`);
                    // it's a plan
                    field.extensions.grafast = {
                        plan: fieldSpec,
                    };
                }
                else {
                    // it's a spec
                    const grafastExtensions = Object.create(null);
                    field.extensions.grafast = grafastExtensions;
                    if (typeof fieldSpec.resolve === "function") {
                        (0, utils_js_1.exportNameHint)(fieldSpec.resolve, `${typeName}_${fieldName}_resolve`);
                        field.resolve = fieldSpec.resolve;
                    }
                    if (typeof fieldSpec.subscribe === "function") {
                        (0, utils_js_1.exportNameHint)(fieldSpec.subscribe, `${typeName}_${fieldName}_subscribe`);
                        field.subscribe = fieldSpec.subscribe;
                    }
                    if (typeof fieldSpec.plan === "function") {
                        (0, utils_js_1.exportNameHint)(fieldSpec.plan, `${typeName}_${fieldName}_plan`);
                        grafastExtensions.plan = fieldSpec.plan;
                    }
                    if (typeof fieldSpec.subscribePlan === "function") {
                        (0, utils_js_1.exportNameHint)(fieldSpec.subscribePlan, `${typeName}_${fieldName}_subscribePlan`);
                        grafastExtensions.subscribePlan = fieldSpec.subscribePlan;
                    }
                    if (typeof fieldSpec.args === "object" && fieldSpec.args != null) {
                        for (const [argName, argSpec] of Object.entries(fieldSpec.args)) {
                            const arg = field.args.find((arg) => arg.name === argName);
                            if (!arg) {
                                console.warn(`'plans' specified configuration for object type '${typeName}' field '${fieldName}' arg '${argName}', but that arg was not present in the type`);
                                continue;
                            }
                            if (typeof argSpec === "function") {
                                // Invalid
                                throw new Error(`Invalid configuration for plans.${typeName}.${fieldName}.args.${argName} - saw a function, but expected an object with 'inputPlan' (optional) and 'applyPlan' (optional) plans`);
                            }
                            else {
                                if (argSpec) {
                                    (0, utils_js_1.exportNameHint)(argSpec.applyPlan, `${typeName}_${fieldName}_${argName}_applyPlan`);
                                    (0, utils_js_1.exportNameHint)(argSpec.inputPlan, `${typeName}_${fieldName}_${argName}_inputPlan`);
                                }
                                const grafastExtensions = Object.create(null);
                                arg.extensions.grafast = grafastExtensions;
                                Object.assign(grafastExtensions, argSpec);
                            }
                        }
                    }
                }
            }
        }
        else if (isInputObjectType(type)) {
            if (typeof spec !== "object" || !spec) {
                throw new Error(`Invalid input object config for '${typeName}'`);
            }
            const inputSpec = spec;
            const fields = type.getFields();
            for (const [fieldName, fieldSpec] of Object.entries(inputSpec)) {
                const field = fields[fieldName];
                if (!field) {
                    console.warn(`'plans' specified configuration for input object type '${typeName}' field '${fieldName}', but that field was not present in the type`);
                    continue;
                }
                if (typeof fieldSpec === "function") {
                    throw new Error(`Expected input object type '${typeName}' field '${fieldName}' to be an object, but found a function. We don't know if this should be the 'inputPlan' or 'applyPlan' - please supply an object.`);
                }
                else {
                    if (fieldSpec) {
                        (0, utils_js_1.exportNameHint)(fieldSpec.inputPlan, `${typeName}_${fieldName}_inputPlan`);
                        (0, utils_js_1.exportNameHint)(fieldSpec.applyPlan, `${typeName}_${fieldName}_applyPlan`);
                    }
                    // it's a spec
                    const grafastExtensions = Object.create(null);
                    field.extensions.grafast = grafastExtensions;
                    Object.assign(grafastExtensions, fieldSpec);
                }
            }
        }
        else if (isInterfaceType(type) || isUnionType(type)) {
            if (typeof spec !== "object" || !spec) {
                throw new Error(`Invalid interface/union config for '${typeName}'`);
            }
            const polySpec = spec;
            if (polySpec.__resolveType) {
                (0, utils_js_1.exportNameHint)(polySpec.__resolveType, `${typeName}_resolveType`);
                type.resolveType = polySpec.__resolveType;
            }
        }
        else if (isScalarType(type)) {
            if (typeof spec !== "object" || !spec) {
                throw new Error(`Invalid scalar config for '${typeName}'`);
            }
            const scalarSpec = spec;
            if (typeof scalarSpec.serialize === "function") {
                (0, utils_js_1.exportNameHint)(scalarSpec.serialize, `${typeName}_serialize`);
                type.serialize = scalarSpec.serialize;
            }
            if (typeof scalarSpec.parseValue === "function") {
                (0, utils_js_1.exportNameHint)(scalarSpec.parseValue, `${typeName}_parseValue`);
                type.parseValue = scalarSpec.parseValue;
            }
            if (typeof scalarSpec.parseLiteral === "function") {
                (0, utils_js_1.exportNameHint)(scalarSpec.parseLiteral, `${typeName}_parseLiteral`);
                type.parseLiteral = scalarSpec.parseLiteral;
            }
            if (typeof scalarSpec.plan === "function") {
                (0, utils_js_1.exportNameHint)(scalarSpec.plan, `${typeName}_plan`);
                type.extensions.grafast = { plan: scalarSpec.plan };
            }
        }
        else if (isEnumType(type)) {
            if (typeof spec !== "object" || !spec) {
                throw new Error(`Invalid enum config for '${typeName}'`);
            }
            const enumValues = type.getValues();
            for (const [enumValueName, enumValueSpec] of Object.entries(spec)) {
                const enumValue = enumValues.find((val) => val.name === enumValueName);
                if (!enumValue) {
                    console.warn(`'plans' specified configuration for enum type '${typeName}' value '${enumValueName}', but that value was not present in the type`);
                    continue;
                }
                if (typeof enumValueSpec === "function") {
                    (0, utils_js_1.exportNameHint)(enumValueSpec, `${typeName}_${enumValueName}_applyPlan`);
                    // It's a plan
                    enumValue.extensions.grafast = {
                        applyPlan: enumValueSpec,
                    };
                }
                else if (typeof enumValueSpec === "object" && enumValueSpec != null) {
                    // It's a full spec
                    if (enumValueSpec.applyPlan) {
                        (0, utils_js_1.exportNameHint)(enumValueSpec.applyPlan, `${typeName}_${enumValueName}_applyPlan`);
                        enumValue.extensions.grafast = {
                            applyPlan: enumValueSpec.applyPlan,
                        };
                    }
                    if ("value" in enumValueSpec) {
                        enumValue.value = enumValueSpec.value;
                    }
                }
                else {
                    // It must be the value
                    enumValue.value = enumValueSpec;
                }
            }
        }
        else {
            const never = type;
            console.error(`Unhandled type ${never}`);
        }
    }
    return schema;
}
exports.makeGrafastSchema = makeGrafastSchema;
//# sourceMappingURL=makeGrafastSchema.js.map